function [alpha, beta, logl, dens] = hmm_fb (X, A, pi0, mu, Sigma, QUIET)

%hmm_fb    Implements the forward-backward recursions (with scaling).
%	Use: [alpha,beta,logl,dens] = hmm_fb(X,A,pi0,mu,Sigma) where
%	alpha and beta are the forward and backward variables and logl
%       is the log-likelihood. Alpha is scaled to sum to 1 at each index
%       and coincides with the filtering probability. dens contains the
%       values of all Gaussian densities for each time index (usefull for
%       the estimation of the transition probabilities).

% H2M Toolbox, $Revision: 2.3 $        
% Olivier Capp, 11/03/96 - $Date: 2004/03/15 14:01:01 $
% http://www.tsi.enst.fr/~cappe/h2m/

% Input args.
error(nargchk(5, 6, nargin));
if (nargin < 6)
  QUIET = 0;
end
[N, p, DIAG_COV] = hmm_chk(A, pi0, mu, Sigma);
[T, Nc] = size(X);
if (Nc ~= p)
  error('Observation vectors have an incorrect dimension.');
end

% Compute density values
dens = gauseval(X, mu, Sigma, QUIET);

% Forward pass
if (~QUIET)
  fprintf(1, 'Forward pass...'); time = cputime;
end
alpha = zeros(T, N);
scale = zeros(T, 1);
alpha(1,:) = pi0.*dens(1,:);
scale(1) = sum(alpha(1,:));
alpha(1,:) = alpha(1,:) / scale(1);
for i=2:T
  alpha(i,:) = (alpha(i-1,:) * A) .* dens(i,:);
  % Systematic scaling
  scale(i) = sum(alpha(i,:));
  alpha(i,:) = alpha(i,:) / scale(i);
end
% This is not computationnaly efficient but log(prod()) won't work here !
logl = sum(log(scale));
if (~QUIET)
  time = cputime - time;
  fprintf(1, ' (%.2f s)\n', time);
end

% Backward pass
if (~QUIET)
  fprintf(1, 'Backward pass...'); time = cputime;
end
% Scale the backward variable with the forward scale factors (this ensures
% that the reestimation of the transition matrix is correct)
beta = zeros(T, N);
beta(T, :) = ones(1, N) / scale(T);
for i=(T-1):-1:1
  beta(i, :) = (beta(i+1,:).* dens(i+1,:)) * A';
  % Apply scaling factor
  beta(i,:) = beta(i,:) / scale(i);
end
if (~QUIET)
  time = cputime - time; fprintf(1, ' (%.2f s)\n',time);
end
